/* Direct3D 8 private include file
 */

#ifndef __WINE_D3DX8_PRIVATE_H
#define __WINE_D3DX8_PRIVATE_H

/* THIS FILE MUST NOT CONTAIN X11 or MESA DEFINES */
#include "config.h"
#include "wine_gl.h"

#include "d3d8.h"

/* Device caps */
#define MAX_STREAMS       1
#define MAX_ACTIVE_LIGHTS 8
#define MAX_CLIPPLANES    D3DMAXUSERCLIPPLANES

/* Other useful values */
#define HIGHEST_RENDER_STATE 174
#define HIGHEST_TEXTURE_STATE 29
#define HIGHEST_TRANSFORMSTATE 512
#define D3DSBT_RECORDED 0xfffffffe

/* Used for CreateStateBlock */
#define NUM_SAVEDPIXELSTATES_R     38
#define NUM_SAVEDPIXELSTATES_T     27
#define NUM_SAVEDVERTEXSTATES_R    33
#define NUM_SAVEDVERTEXSTATES_T    2

/* Direct38 Interfaces: */
typedef struct IDirect3DBaseTexture8Impl IDirect3DBaseTexture8Impl;
typedef struct IDirect3DVolumeTexture8Impl IDirect3DVolumeTexture8Impl;
typedef struct IDirect3D8Impl IDirect3D8Impl;
typedef struct IDirect3DDevice8Impl IDirect3DDevice8Impl;
typedef struct IDirect3DTexture8Impl IDirect3DTexture8Impl;
typedef struct IDirect3DCubeTexture8Impl IDirect3DCubeTexture8Impl;
typedef struct IDirect3DIndexBuffer8Impl IDirect3DIndexBuffer8Impl;
typedef struct IDirect3DSurface8Impl IDirect3DSurface8Impl;
typedef struct IDirect3DSwapChain8Impl IDirect3DSwapChain8Impl;
typedef struct IDirect3DResource8Impl IDirect3DResource8Impl;
typedef struct IDirect3DVolume8Impl IDirect3DVolume8Impl;
typedef struct IDirect3DVertexBuffer8Impl IDirect3DVertexBuffer8Impl;

/* State Block for Begin/End/Capture/Create/Apply State Block   */
/*   Note: Very long winded but I do not believe gl Lists will  */
/*   resolve everything we need, so doing it manually for now   */
typedef struct SAVEDSTATES {
        BOOL                      lightEnable[MAX_ACTIVE_LIGHTS];
        BOOL                      Indices;
        BOOL                      lights[MAX_ACTIVE_LIGHTS];
        BOOL                      material;
        BOOL                      stream_source[MAX_STREAMS];
        BOOL                      textures[8];
        BOOL                      transform[HIGHEST_TRANSFORMSTATE];
        BOOL                      viewport;
        BOOL                      vertexShader;
        BOOL                      pixelShader;
        BOOL                      renderstate[HIGHEST_RENDER_STATE];
        BOOL                      texture_state[8][HIGHEST_TEXTURE_STATE];
        BOOL                      clipplane[MAX_CLIPPLANES];
} SAVEDSTATES;
    
typedef struct STATEBLOCK {
    
    D3DSTATEBLOCKTYPE             blockType;

    SAVEDSTATES                   Changed;
    SAVEDSTATES                   Set;

    /* Light Enable */
    BOOL                      lightEnable[MAX_ACTIVE_LIGHTS];

    /* ClipPlane */
    double                    clipplane[MAX_CLIPPLANES][4];

    /* Indices */
    IDirect3DIndexBuffer8*    pIndexData;
    UINT                      baseVertexIndex;

    /* Lights */
    D3DLIGHT8                 lights[MAX_ACTIVE_LIGHTS];

    /* Material */
    D3DMATERIAL8              material;

    /* Pixel Shader */
    DWORD                     PixelShader;

    /* TODO: Pixel Shader Constant */

    /* RenderState */
    DWORD                     renderstate[HIGHEST_RENDER_STATE];

    /* Stream Source */
    UINT                      stream_stride[MAX_STREAMS];
    IDirect3DVertexBuffer8   *stream_source[MAX_STREAMS];

    /* Texture */
    IDirect3DBaseTexture8    *textures[8];

    /* Texture State Stage */
    DWORD                     texture_state[8][HIGHEST_TEXTURE_STATE];

    /* Transform */
    D3DMATRIX                 transforms[HIGHEST_TRANSFORMSTATE];

    /* ViewPort */
    D3DVIEWPORT8              viewport;

    /* Vertex Shader */
    DWORD                     VertexShader;


    /* TODO: Vertex Shader Constant */

} STATEBLOCK;

/* Globals */
extern DWORD SavedPixelStates_R[NUM_SAVEDPIXELSTATES_R];
extern DWORD SavedPixelStates_T[NUM_SAVEDPIXELSTATES_T];
extern DWORD SavedVertexStates_R[NUM_SAVEDVERTEXSTATES_R];
extern DWORD SavedVertexStates_T[NUM_SAVEDVERTEXSTATES_T];


#include "IDirect3D8.h"
#include "IDirect3DBaseTexture8.h"
#include "IDirect3DDevice8.h"
#include "IDirect3DResource8.h"
#include "IDirect3DVertexBuffer8.h"
#include "IDirect3DVolume8.h"
#include "IDirect3DSwapChain8.h"
#include "IDirect3DSurface8.h"
#include "IDirect3DIndexBuffer8.h"
#include "IDirect3DCubeTexture8.h"
#include "IDirect3DTexture8.h"
#include "IDirect3DVolumeTexture8.h"

/* External prototypes */
//BOOL D3DRAW_HAL_Init(HINSTANCE, DWORD, LPVOID);                          
void CreateStateBlock(STATEBLOCK *pSB);

extern Display *gdi_display;
extern Window root_window;
extern Window d3dhack(HWND hwnd);

/* Utility calls */
#define conv_mat(mat,gl_mat)			\
{                                               \
    TRACE("%f %f %f %f\n", (mat)->_11, (mat)->_12, (mat)->_13, (mat)->_14); \
    TRACE("%f %f %f %f\n", (mat)->_21, (mat)->_22, (mat)->_23, (mat)->_24); \
    TRACE("%f %f %f %f\n", (mat)->_31, (mat)->_32, (mat)->_33, (mat)->_34); \
    TRACE("%f %f %f %f\n", (mat)->_41, (mat)->_42, (mat)->_43, (mat)->_44); \
    memcpy(gl_mat, (mat), 16 * sizeof(float));      \
};

#define checkGLcall(A) \
{ \
    GLint err = glGetError();   \
    if (err != GL_NO_ERROR) { \
       FIXME(">>>>>>>>>>>>>>>>> %x from %s @ %s / %d\n", err, A, __FILE__, __LINE__); \
    } else { \
       TRACE("%s call ok %s / %d\n", A, __FILE__, __LINE__); \
    } \
} 



#endif /* __WINE_D3DX8_PRIVATE_H */
